@@ -52,7 +52,7 @@ Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 =cut
 
-# $Id: docbook2man-spec.pl,v 1.1 2000/07/21 20:22:30 rosalia Exp $
+# $Id: docbook2man-spec.pl,v 1.4 2002/04/15 16:32:33 twaugh Exp $
 
 use SGMLS;			# Use the SGMLS package.
 use SGMLS::Output;		# Use stack-based output.
@@ -412,12 +412,49 @@ man_sgml('</CMDSYNOPSIS>', "\n\n");
 man_sgml('<FUNCPROTOTYPE>', "\n.sp\n");
 
 # Arguments to functions.  This is C convention.
-man_sgml('<PARAMDEF>', '(');
-man_sgml('</PARAMDEF>', ");\n");
-man_sgml('<VOID>', "(void);\n");
+sub paramdef
+{
+	if($_[0]->parent->ext->{'inparams'}) {
+		output ', ';
+	} else {
+		output ' (';
+		$_[0]->parent->ext->{'inparams'} = 1;
+	}
+}
+man_sgml('<PARAMDEF>', \&paramdef);
+man_sgml('</FUNCPROTOTYPE>', ");\n");
+man_sgml('<VOID>', "(void");
+man_sgml('<VARARGS>', "(...");
 
 
 
+sub group_start
+{
+	if(not $_[0]->parent->in('TERM')) {
+		if($_[0]->attribute('CHOICE')->value =~ /opt/i) {
+			output ' [';
+		} elsif($_[0]->attribute('CHOICE')->value =~ /req/i) {
+			output ' {';
+		}
+	}
+	$_[0]->ext->{'count'} = 1;
+}
+sub group_end
+{
+	if($_[0]->attribute('REP')->value =~ /^Repeat/i) {
+		italic_on();
+		output ' ...';
+		font_off();
+	}
+	if(not $_[0]->parent->in('TERM')) {
+		if($_[0]->attribute('CHOICE')->value =~ /opt/i) {
+			output ' ]';
+		} elsif($_[0]->attribute('CHOICE')->value =~ /req/i) {
+			output ' }';
+		}
+	}
+}
+
 sub arg_start
 {
 	# my $choice = $_[0]->attribute('CHOICE')->value;
@@ -427,7 +464,10 @@ sub arg_start
 	# so we have to add it after each component.
 	output ' ';
 
-	if($_[0]->attribute('CHOICE')->value =~ /opt/i) {
+	if($_[0]->in('GROUP')) {
+		output '| ' if $_[0]->parent->ext->{'count'} > 1;
+		$_[0]->parent->ext->{'count'}++;
+	} elsif($_[0]->attribute('CHOICE')->value =~ /opt/i) {
 		output '[ ';
 	}
 	bold_on();
@@ -440,21 +480,21 @@ sub arg_end
 		output ' ...';
 		font_off();
 	}
-	if($_[0]->attribute('CHOICE')->value =~ /opt/i) {
-		output '] ';
+	if($_[0]->attribute('CHOICE')->value =~ /opt/i and
+           not $_[0]->in('GROUP')) {
+		output ' ]';
 	}
 }
 
 sgml('<ARG>', \&arg_start);
 sgml('</ARG>', \&arg_end);
-sgml('<GROUP>', \&arg_start);
-sgml('</GROUP>', \&arg_end);
+sgml('<GROUP>', \&group_start);
+sgml('</GROUP>', \&group_end);
 
 sgml('<OPTION>', \&bold_on);
 sgml('</OPTION>', \&font_off);
 
-# FIXME: This is one _blank_ line.
-man_sgml('<SBR>', "\n\n");
+man_sgml('<SBR>', "\n   ");
 
 
 ########################################################################
@@ -991,7 +1031,7 @@ sgml('<LISTITEM>', sub {
 	# Assume Arabic numeration for now.
 	elsif($_[0]->in('ORDEREDLIST')) {
 		output "\n" unless $newline_last++;
-		output ".TP ", $_[0]->parent->ext->{'count'}++, ". \n";
+		output ".TP 3\n", $_[0]->parent->ext->{'count'}++, ". \n";
 	}
 	
 	$_[0]->ext->{'nobreak'} = 1;
@@ -1153,7 +1193,11 @@ sgml('re', sub
 	$newline_last = 1;
 });
 
-sgml('sdata',sub { die "Unknown SDATA: " . $_[0]; });
+sgml('sdata',sub
+{
+	if($_[0] =~ /\[minus \]/) { output "-"; }
+	else { die "Unknown SDATA: " . $_[0]; }
+});
 sgml('pi',sub { die "Unknown processing instruction: " . $_[0]; });
 sgml('entity',sub { die "Unknown external entity: " . $_[0]->name; });
 sgml('start_subdoc',sub { die "Unknown subdoc entity: " . $_[0]->name; });
